'use strict';

var NullFactory = require('webpack/lib/NullFactory');
var LocalModuleDependency = require('webpack/lib/dependencies/LocalModuleDependency');
var ConstDependency = require('webpack/lib/dependencies/ConstDependency');
var ContextElementDependency = require('webpack/lib/dependencies/ContextElementDependency');
var OpenUI5DefineDependency = require('./OpenUI5DefineDependency');
var OpenUI5DefineDependencyParserPlugin = require('./OpenUI5DefineDependencyParserPlugin');
var OpenUI5LazyInstanceDependency = require('./OpenUI5LazyInstanceDependency');
var OpenUI5RequireDependencyParserPlugin = require('./OpenUI5RequireDependencyParserPlugin');
var OpenUI5RequireItemDependency = require('./OpenUI5RequireItemDependency');
var OpenUI5RequireContextDependency = require('./OpenUI5RequireContextDependency');
var OpenUI5ResourceDependencyParserPlugin = require('./OpenUI5ResourceDependencyParserPlugin');
var OpenUI5ResourceDependency = require('./OpenUI5ResourceDependency');
var OpenUI5ResourceModuleFactory = require('./OpenUI5ResourceModuleFactory');

class OpenUI5Plugin {
  constructor(options) {
    this.options = options;
  }

  apply(compiler) {
    var options = this.options;


    var resourceModuleFactory = new OpenUI5ResourceModuleFactory(compiler.resolverFactory);

    compiler.hooks.compilation.tap('OpenUI5Plugin', function (compilation, _ref) {
      var normalModuleFactory = _ref.normalModuleFactory,
          contextModuleFactory = _ref.contextModuleFactory;

      compilation.dependencyFactories.set(OpenUI5RequireItemDependency, normalModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5RequireItemDependency, new OpenUI5RequireItemDependency.Template());

      compilation.dependencyFactories.set(OpenUI5RequireContextDependency, contextModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5RequireContextDependency, new OpenUI5RequireContextDependency.Template());

      compilation.dependencyFactories.set(OpenUI5DefineDependency, new NullFactory());
      compilation.dependencyTemplates.set(OpenUI5DefineDependency, new OpenUI5DefineDependency.Template());

      compilation.dependencyFactories.set(OpenUI5LazyInstanceDependency, normalModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5LazyInstanceDependency, new OpenUI5LazyInstanceDependency.Template());

      compilation.dependencyFactories.set(OpenUI5ResourceDependency, resourceModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5ResourceDependency, new OpenUI5ResourceDependency.Template());

      compilation.dependencyFactories.set(LocalModuleDependency, new NullFactory());
      compilation.dependencyTemplates.set(LocalModuleDependency, new LocalModuleDependency.Template());

      compilation.dependencyFactories.set(ConstDependency, new NullFactory());
      compilation.dependencyTemplates.set(ConstDependency, new ConstDependency.Template());

      compilation.dependencyFactories.set(ContextElementDependency, normalModuleFactory);
      compilation.dependencyFactories.set(ContextElementDependency, normalModuleFactory);

      normalModuleFactory.hooks.parser.for('javascript/auto').tap('OpenUI5Plugin', function (parser) {
        new OpenUI5RequireDependencyParserPlugin(options).apply(parser);
        new OpenUI5DefineDependencyParserPlugin(options).apply(parser);
        new OpenUI5ResourceDependencyParserPlugin(options).apply(parser);
      });
    });
  }
}
module.exports = OpenUI5Plugin;